# © 2024 Le Filament (<http://www.le-filament.com>)
# License AGPL-3.0 or later (http://www.gnu.org/licenses/agpl.html).

from odoo import _, api, fields, models
from odoo.exceptions import AccessError, UserError


class FinancialContract(models.AbstractModel):
    _name = "financial.contract"
    _description = "Financial Contract"
    _order = "create_date desc"

    name = fields.Char(compute="_compute_name")
    number = fields.Char("Numéro", readonly=True)
    partner_id = fields.Many2one(
        comodel_name="res.partner",
        string="Client",
        required=True,
    )
    active = fields.Boolean(
        default=True,
    )
    amount = fields.Monetary("Montant")
    company_id = fields.Many2one(
        comodel_name="res.company",
        string="Société",
        default=lambda self: self.env.company,
    )
    currency_id = fields.Many2one(
        comodel_name="res.currency", related="company_id.currency_id"
    )
    comment = fields.Text("Commentaire")
    expiration_date = fields.Date("Date d'expiration")

    guarantee_id = fields.Many2one(
        comodel_name="financial.external.guarantee",
        string="Garantie",
        ondelete="restrict",
    )
    funding_id = fields.Many2one(
        comodel_name="financial.funding",
        string="Fonds financeur",
        ondelete="restrict",
    )
    state = fields.Selection(
        [
            ("init", "Initialisation"),
            ("offer", "Offre"),
            ("proposal", "Proposition"),
            ("contract", "Signé"),
            ("done", "Terminé"),
            ("cancel", "Annulé"),
        ],
        string="Statut",
        tracking=1,
        default="init",
    )

    # ------------------------------------------------------
    # Constrains
    # ------------------------------------------------------
    @api.constrains("state", "company_id")
    def _check_user_company(self):
        for contract in self:
            if contract.state in ["proposal", "contract", "done", "cancel"]:
                if contract.company_id not in self.env.user.company_ids:
                    raise AccessError(
                        _("Vous n'êtes pas autorisé à modifier ce contrat.")
                    )

    # ------------------------------------------------------
    # Computed fields / Search Fields
    # ------------------------------------------------------
    def _compute_name(self):
        for contract in self:
            contract.name = (
                f"{contract.number or '#'} - " f"{contract.partner_id.name} "
            )

    # ------------------------------------------------------
    # Onchange
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Actions
    # ------------------------------------------------------

    # ------------------------------------------------------
    # CRUD (Override ORM)
    # ------------------------------------------------------
    def unlink(self):
        """
        Restrict contract deletion depending on state
        """
        for contract in self:
            if contract.state != "init":
                raise UserError(
                    _("Seuls les deals à l'état Projet peuvent être supprimés")
                )
        return super().unlink()

    # ------------------------------------------------------
    # Override ORM
    # ------------------------------------------------------

    # ------------------------------------------------------
    # Business functions
    # ------------------------------------------------------
